<?php

namespace Aedes\EveryPay\Model\Form;

class FormDataBuilder
{
    /**
     * @var \Aedes\EveryPay\Model\Config
     */
    protected $config;

    /**
     * @var \Magento\Framework\Locale\Resolver
     */
    protected $locale;

    /**
     * @var \Aedes\EveryPay\Model\Hmac\HmacManagement
     */
    protected $hmac;

    /**
     * @var ApiData
     */
    protected $apiData;

    /**
     * @var OrderData
     */
    protected $orderData;

    /**
     * @var TokenData
     */
    protected $tokenData;

    /**
     * @param \Magento\Framework\Locale\Resolver $locale
     * @param \Aedes\EveryPay\Model\Config $config
     * @param \Aedes\EveryPay\Model\Hmac\HmacManagement $hmac
     * @param ApiData $apiData
     * @param OrderData $orderData
     * @param TokenData $tokenData
     */
    public function __construct(
        \Magento\Framework\Locale\Resolver $locale,
        \Aedes\EveryPay\Model\Config $config,
        \Aedes\EveryPay\Model\Hmac\HmacManagement $hmac,
        ApiData $apiData,
        OrderData $orderData,
        TokenData $tokenData
    ) {
        $this->locale = $locale;
        $this->config = $config;
        $this->hmac = $hmac;
        $this->apiData = $apiData;
        $this->orderData = $orderData;
        $this->tokenData = $tokenData;
    }

    /**
     * @param int $orderId
     * @param null|array $tokenData
     * @return array
     */
    public function getFields($orderId, $tokenData = null)
    {
        if ($this->config->getCcTokenEnabled() && $tokenData != null) {
            $tokenFields = $this->tokenData->getFields($tokenData);
        } else {
            $tokenFields = [];
        }

        $data = $this->hmac->appendHmacFields(
            array_merge(
                $this->apiData->getFields($orderId),
                $this->orderData->getFields($orderId),
                $tokenFields
            )
        );

        $data['locale'] = $this->resolveLocale();
        return $data;
    }

    /**
     * @return string
     */
    protected function resolveLocale()
    {
        $locale = $this->locale->getLocale();
        $locale = substr($locale, 0, strpos($locale, '_'));

        if (!in_array($locale, $this->config->getSupportedLocales())) {
            $locale =  $this->config->getDefaultLocale();
        }

        return $locale;
    }
}