<?php

namespace Aedes\EveryPay\Observer;

use Magento\Framework\Event\ObserverInterface;

class OrderEmailCancel implements ObserverInterface
{
    /**
     * Disallow sending the order email after placing the order
     *
     * @param \Magento\Framework\Event\Observer $observer
     * @return void
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        /** @var  \Magento\Quote\Model\Quote $quote */
        $quote = $observer->getEvent()->getQuote();
        /** @var  \Magento\Sales\Model\Order $order */
        $order = $observer->getEvent()->getOrder();

        if ($this->hasEverypayPaymentMethod($quote)) {
            $order->setCanSendNewEmailFlag(false);
        }
    }

    /**
     * Checks if quote that is being submitted has EveryPay payment method
     *
     * @param \Magento\Quote\Model\Quote $quote
     * @return bool
     */
    protected function hasEverypayPaymentMethod($quote)
    {
        return $quote->getPayment()->getMethodInstance()->getCode() == \Aedes\EveryPay\Model\EveryPay::CODE;
    }
}